#ifndef	_SYS_IOCTL_H_
#define	_SYS_IOCTL_H_

/*
 * Window/terminal size structure.  This information is stored by the kernel
 * in order to provide a consistent interface, but is not used by the kernel.
 */
struct winsize {
	unsigned short	ws_row;		/* rows, in characters */
	unsigned short	ws_col;		/* columns, in characters */
	unsigned short	ws_xpixel;	/* horizontal size, pixels */
	unsigned short	ws_ypixel;	/* vertical size, pixels */
};

/*
 * Pun for SUN.
 */
struct ttysize {
	unsigned short	ts_lines;
	unsigned short	ts_cols;
	unsigned short	ts_xxx;
	unsigned short	ts_yyy;
};
#define	TIOCGSIZE	TIOCGWINSZ
#define	TIOCSSIZE	TIOCSWINSZ

/*
 * Ioctl's have the command encoded in the lower word, and the size of
 * any in or out parameters in the upper word.  The high 3 bits of the
 * upper word are used to encode the in/out status of the parameter.
 */
#define	IOCPARM_MASK	0x1fff		/* parameter length, at most 13 bits */
#define	IOCPARM_LEN(x)	(((x) >> 16) & IOCPARM_MASK)
#define	IOCBASECMD(x)	((x) & ~(IOCPARM_MASK << 16))
#define	IOCGROUP(x)	(((x) >> 8) & 0xff)

#define	IOCPARM_MAX	NBPG		/* max size of ioctl, mult. of NBPG */
#define	IOC_VOID	0x20000000	/* no parameters */
#define	IOC_OUT		0x40000000	/* copy out parameters */
#define	IOC_IN		0x80000000	/* copy in parameters */
#define	IOC_INOUT	(IOC_IN|IOC_OUT)
#define	IOC_DIRMASK	0xe0000000	/* mask for IN/OUT/VOID */

#define	__IOC(inout,group,num,len) \
	(inout | ((len & IOCPARM_MASK) << 16) | ((group) << 8) | (num))
#define	__IO(g,n)	__IOC(IOC_VOID,	(g), (n), 0)


#define	__IOR(g,n,t)	__IOC(IOC_OUT,	(g), (n), sizeof(t))
#define	__IOW(g,n,t)	__IOC(IOC_IN,	(g), (n), sizeof(t))
/* this should be __IORW, but stdio got there first */
#define	__IOWR(g,n,t)	__IOC(IOC_INOUT,	(g), (n), sizeof(t))

#define	TIOCMODG	__IOR('t', 3, int)	/* get modem control state */
#define	TIOCMODS	__IOW('t', 4, int)	/* set modem control state */
#define		TIOCM_LE	0001		/* line enable */
#define		TIOCM_DTR	0002		/* data terminal ready */
#define		TIOCM_RTS	0004		/* request to send */
#define		TIOCM_ST	0010		/* secondary transmit */
#define		TIOCM_SR	0020		/* secondary receive */
#define		TIOCM_CTS	0040		/* clear to send */
#define		TIOCM_CAR	0100		/* carrier detect */
#define		TIOCM_CD	TIOCM_CAR
#define		TIOCM_RNG	0200		/* ring */
#define		TIOCM_RI	TIOCM_RNG
#define		TIOCM_DSR	0400		/* data set ready */
						/* 8-10 compat */
#define	TIOCEXCL	__IO('t', 13)		/* set exclusive use of tty */
#define	TIOCNXCL	__IO('t', 14)		/* reset exclusive use of tty */
						/* 15 unused */
#define	TIOCFLUSH	__IOW('t', 16, int)	/* flush buffers */
						/* 17-18 compat */
#define	TIOCGETA	__IOR('t', 19, struct termios) /* get termios struct */
#define	TIOCSETA	__IOW('t', 20, struct termios) /* set termios struct */
#define	TIOCSETAW	__IOW('t', 21, struct termios) /* drain output, set */
#define	TIOCSETAF	__IOW('t', 22, struct termios) /* drn out, fls in, set */
#define	TIOCGETD	__IOR('t', 26, int)	/* get line discipline */
#define	TIOCSETD	__IOW('t', 27, int)	/* set line discipline */
						/* 127-124 compat */
#define	TIOCSBRK	__IO('t', 123)		/* set break bit */
#define	TIOCCBRK	__IO('t', 122)		/* clear break bit */
#define	TIOCSDTR	__IO('t', 121)		/* set data terminal ready */
#define	TIOCCDTR	__IO('t', 120)		/* clear data terminal ready */
#define	TIOCGPGRP	__IOR('t', 119, int)	/* get pgrp of tty */
#define	TIOCSPGRP	__IOW('t', 118, int)	/* set pgrp of tty */
						/* 117-116 compat */
#define	TIOCOUTQ	__IOR('t', 115, int)	/* output queue size */
#define	TIOCSTI		__IOW('t', 114, char)	/* simulate terminal input */
#define	TIOCNOTTY	__IO('t', 113)		/* void tty association */
#define	TIOCPKT		__IOW('t', 112, int)	/* pty: set/clear packet mode */
#define		TIOCPKT_DATA		0x00	/* data packet */
#define		TIOCPKT_FLUSHREAD	0x01	/* flush packet */
#define		TIOCPKT_FLUSHWRITE	0x02	/* flush packet */
#define		TIOCPKT_STOP		0x04	/* stop output */
#define		TIOCPKT_START		0x08	/* start output */
#define		TIOCPKT_NOSTOP		0x10	/* no more ^S, ^Q */
#define		TIOCPKT_DOSTOP		0x20	/* now do ^S ^Q */
#define		TIOCPKT_IOCTL		0x40	/* state change of pty driver */
#define	TIOCSTOP	__IO('t', 111)		/* stop output, like ^S */
#define	TIOCSTART	__IO('t', 110)		/* start output, like ^Q */
#define	TIOCMSET	__IOW('t', 109, int)	/* set all modem bits */
#define	TIOCMBIS	__IOW('t', 108, int)	/* bis modem bits */
#define	TIOCMBIC	__IOW('t', 107, int)	/* bic modem bits */
#define	TIOCMGET	__IOR('t', 106, int)	/* get all modem bits */
#define	TIOCREMOTE	__IOW('t', 105, int)	/* remote input editing */
#define	TIOCGWINSZ	__IOR('t', 104, struct winsize)	/* get window size */
#define	TIOCSWINSZ	__IOW('t', 103, struct winsize)	/* set window size */
#define	TIOCUCNTL	__IOW('t', 102, int)	/* pty: set/clr usr cntl mode */
#define		UIOCCMD(n)	__IO('u', n)		/* usr cntl op "n" */
#define TIOCSTAT	__IOW('t', 101, int)	/* generate status message */
#define	TIOCCONS	__IOW('t', 98, int)		/* become virtual console */
#define	TIOCSCTTY	__IO('t', 97)		/* become controlling tty */
#define	TIOCEXT		__IOW('t', 96, int)	/* pty: external processing */
#define	TIOCSIG		__IO('t', 95)		/* pty: generate signal */
#define	TIOCDRAIN	__IO('t', 94)		/* wait till output drained */
#define TIOCGFLAGS	__IOR('t', 93, int)	/* set device flags */
#define TIOCSFLAGS	__IOW('t', 92, int)	/* set device flags */
#define		TIOCFLAG_SOFTCAR	0x01	/* ignore hardware carrier */
#define		TIOCFLAG_CLOCAL		0x02	/* set clocal on open */
#define		TIOCFLAG_CRTSCTS	0x04	/* set crtscts on open */
#define		TIOCFLAG_MDMBUF		0x08	/* set mdmbuf on open */

#define	TTYDISC		0		/* termios tty line discipline */
#define	TABLDISC	3		/* tablet discipline */
#define	SLIPDISC	4		/* serial IP discipline */
#define	PPPDISC		5		/* ppp discipline */


#define	FIOCLEX		__IO('f', 1)		/* set close on exec on fd */
#define	FIONCLEX	__IO('f', 2)		/* remove close on exec */
#define	FIONREAD	__IOR('f', 127, int)	/* get # bytes to read */
#define	FIONBIO		__IOW('f', 126, int)	/* set/clear non-blocking i/o */
#define	FIOASYNC	__IOW('f', 125, int)	/* set/clear async i/o */
#define	FIOSETOWN	__IOW('f', 124, int)	/* set owner */
#define	FIOGETOWN	__IOR('f', 123, int)	/* get owner */

/* socket i/o controls */
#define	SIOCSHIWAT	__IOW('s',  0, int)		/* set high watermark */
#define	SIOCGHIWAT	__IOR('s',  1, int)		/* get high watermark */
#define	SIOCSLOWAT	__IOW('s',  2, int)		/* set low watermark */
#define	SIOCGLOWAT	__IOR('s',  3, int)		/* get low watermark */
#define	SIOCATMARK	__IOR('s',  7, int)		/* at oob mark? */
#define	SIOCSPGRP	__IOW('s',  8, int)		/* set process group */
#define	SIOCGPGRP	__IOR('s',  9, int)		/* get process group */

#define	SIOCADDRT	__IOW('r', 10, struct ortentry)	/* add route */
#define	SIOCDELRT	__IOW('r', 11, struct ortentry)	/* delete route */

#define	SIOCSIFADDR	__IOW('i', 12, struct ifreq)	/* set ifnet address */
#define	OSIOCGIFADDR	__IOWR('i',13, struct ifreq)	/* get ifnet address */
#define	SIOCGIFADDR	__IOWR('i',33, struct ifreq)	/* get ifnet address */
#define	SIOCSIFDSTADDR	__IOW('i', 14, struct ifreq)	/* set p-p address */
#define	OSIOCGIFDSTADDR	__IOWR('i',15, struct ifreq)	/* get p-p address */
#define	SIOCGIFDSTADDR	__IOWR('i',34, struct ifreq)	/* get p-p address */
#define	SIOCSIFFLAGS	__IOW('i', 16, struct ifreq)	/* set ifnet flags */
#define	SIOCGIFFLAGS	__IOWR('i',17, struct ifreq)	/* get ifnet flags */
#define	OSIOCGIFBRDADDR	__IOWR('i',18, struct ifreq)	/* get broadcast addr */
#define	SIOCGIFBRDADDR	__IOWR('i',35, struct ifreq)	/* get broadcast addr */
#define	SIOCSIFBRDADDR	__IOW('i',19, struct ifreq)	/* set broadcast addr */
#define	OSIOCGIFCONF	__IOWR('i',20, struct ifconf)	/* get ifnet list */
#define	SIOCGIFCONF	__IOWR('i',36, struct ifconf)	/* get ifnet list */
#define	OSIOCGIFNETMASK	__IOWR('i',21, struct ifreq)	/* get net addr mask */
#define	SIOCGIFNETMASK	__IOWR('i',37, struct ifreq)	/* get net addr mask */
#define	SIOCSIFNETMASK	__IOW('i',22, struct ifreq)	/* set net addr mask */
#define	SIOCGIFMETRIC	__IOWR('i',23, struct ifreq)	/* get IF metric */
#define	SIOCSIFMETRIC	__IOW('i',24, struct ifreq)	/* set IF metric */
#define	SIOCDIFADDR	__IOW('i',25, struct ifreq)	/* delete IF addr */
#define	SIOCAIFADDR	__IOW('i',26, struct ifaliasreq)	/* add/chg IF alias */

#define	SIOCSARP	__IOW('i', 30, struct arpreq)	/* set arp entry */
#define	OSIOCGARP	__IOWR('i',31, struct arpreq)	/* get arp entry */
#define	SIOCGARP	__IOWR('i',38, struct arpreq)	/* get arp entry */
#define	SIOCDARP	__IOW('i', 32, struct arpreq)	/* delete arp entry */

#define	SIOCADDMULTI	__IOW('i', 49, struct ifreq)	/* add m'cast addr */
#define	SIOCDELMULTI	__IOW('i', 50, struct ifreq)	/* del m'cast addr */

#define	SIOCSIFMTU	__IOW('i', 127, struct ifreq)	/* set ifnet mtu */
#define	SIOCGIFMTU	__IOWR('i',126, struct ifreq)	/* get ifnet mtu */
#define	SIOCSIFASYNCMAP	__IOW('i', 125, struct ifreq)	/* set ppp asyncmap */
#define	SIOCGIFASYNCMAP	__IOWR('i',124, struct ifreq)	/* get ppp asyncmap */

int	ioctl (int, unsigned long, ...);


#endif /* !_SYS_IOCTL_H_ */


#if defined(USE_OLD_TTY) || defined(KERNEL)
#include <sys/ioctl_compat.h>
#endif
