#ifndef _SYS_TERMIOS_H_
#define _SYS_TERMIOS_H_


/* 
 * Special Control Characters 
 *
 * Index into c_cc[] character array.
 *
 *	Name	     Subscript	Enabled by 
 */
#define	VEOF		0	/* ICANON */
#define	VEOL		1	/* ICANON */
#define	VEOL2		2	/* ICANON */
#define	VERASE		3	/* ICANON */
#define VWERASE 	4	/* ICANON */
#define VKILL		5	/* ICANON */
#define	VREPRINT 	6	/* ICANON */
/*			7	   spare 1 */
#define VINTR		8	/* ISIG */
#define VQUIT		9	/* ISIG */
#define VSUSP		10	/* ISIG */
#define VDSUSP		11	/* ISIG */
#define VSTART		12	/* IXON, IXOFF */
#define VSTOP		13	/* IXON, IXOFF */
#define	VLNEXT		14	/* IEXTEN */
#define	VDISCARD	15	/* IEXTEN */
#define VMIN		16	/* !ICANON */
#define VTIME		17	/* !ICANON */
#define VSTATUS		18	/* ICANON */
/*			19	   spare 2 */
#define	NCCS		20

#ifdef _POSIX_VDISABLE
#undef _POSIX_VDISABLE
#endif
#define _POSIX_VDISABLE	((unsigned char)'\377')


/*
 * Input flags - software input processing
 */
#define	IGNBRK		0x00000001	/* ignore BREAK condition */
#define	BRKINT		0x00000002	/* map BREAK to SIGINTR */
#define	IGNPAR		0x00000004	/* ignore (discard) parity errors */
#define	PARMRK		0x00000008	/* mark parity and framing errors */
#define	INPCK		0x00000010	/* enable checking of parity errors */
#define	ISTRIP		0x00000020	/* strip 8th bit off chars */
#define	INLCR		0x00000040	/* map NL into CR */
#define	IGNCR		0x00000080	/* ignore CR */
#define	ICRNL		0x00000100	/* map CR to NL (ala CRMOD) */
#define	IXON		0x00000200	/* enable output flow control */
#define	IXOFF		0x00000400	/* enable input flow control */
#define	IXANY		0x00000800	/* any char will restart after stop */
#define	IUCLC		0x00001000	/* upper character to lower character */
#define	IMAXBEL		0x00002000	/* ring bell on input queue full */


/*
 * Output flags - software output processing
 */
#define	OPOST		0x00000001
#define	OLCUC		0x00000002
#define	ONLCR		0x00000004
#define	OCRNL		0x00000008
#define	ONOEOT		0x00000010
#define	ONOCR		0x00000020
#define	ONLRET		0x00000040
#define	OFILL		0x00000080
#define	OFDEL		0x00000100
#define OXTABS		0x00000c00

#define	NLDLY		0x00000300
#define	NL0		0x00000100
#define	NL1		0x00000200
#define	TABDLY		0x00001c00
#define	TAB0		0x00000000
#define	TAB1		0x00000400
#define	TAB2		0x00000800
#define	TAB3		0x00001000
#define	CRDLY		0x000E0000
#define	CR0		0x00000000
#define	CR1		0x00020000
#define	CR2		0x00040000
#define	CR3		0x00080000
#define	BSDLY		0x00800000
#define	BS0		0x00000000
#define	BS1		0x00800000
#define	VTDLY		0x01000000
#define	VT0		0x00000000
#define	VT1		0x01000000
#define	FFDLY		0x02000000
#define	FF0		0x00000000
#define	FF1		0x02000000




/*
 * Control flags - hardware control of terminal
 */
#define	CIGNORE		0x00000001	/* ignore control flags */
#define	CSIZE		0x00000300	/* character size mask */
#define	CS5		0x00000000	/* 5 bits (pseudo) */
#define	CS6		0x00000100	/* 6 bits */
#define	CS7		0x00000200	/* 7 bits */
#define	CS8		0x00000300	/* 8 bits */
#define	CSTOPB		0x00000400	/* send 2 stop bits */
#define	CREAD		0x00000800	/* enable receiver */
#define	PARENB		0x00001000	/* parity enable */
#define	PARODD		0x00002000	/* odd parity, else even */
#define	HUPCL		0x00004000	/* hang up on last close */
#define	CLOCAL		0x00008000	/* ignore modem status lines */
#define	CRTSCTS		0x00010000	/* RTS/CTS full-duplex flow control */
#define	CRTS_IFLOW	CRTSCTS		/* XXX compat */
#define	CCTS_OFLOW	CRTSCTS		/* XXX compat */
#define	MDMBUF		0x00100000	/* DTR/DCD hardware flow control */
#define	CHWFLOW		(MDMBUF|CRTSCTS) /* all types of hw flow control */


/* 
 * "Local" flags - dumping ground for other state
 *
 * Warning: some flags in this structure begin with
 * the letter "I" and look like they belong in the
 * input flag.
 */

#define	ECHOKE		0x00000001	/* visual erase for line kill */
#define	ECHOE		0x00000002	/* visually erase chars */
#define	ECHOK		0x00000004	/* echo NL after line kill */
#define	ECHO		0x00000008	/* enable echoing */
#define	ECHONL		0x00000010	/* echo NL even if ECHO is off */
#define	ECHOPRT		0x00000020	/* visual erase mode for hardcopy */
#define	ECHOCTL		0x00000040	/* echo control chars as ^(Char) */
#define	ISIG		0x00000080	/* enable signals INTR, QUIT, [D]SUSP */
#define	ICANON		0x00000100	/* canonicalize input lines */
#define	ALTWERASE	0x00000200	/* use alternate WERASE algorithm */
#define	IEXTEN		0x00000400	/* enable DISCARD and LNEXT */
#define	EXTPROC 	0x00000800	/* external processing */
#define	XCASE		0x00100000
#define	TOSTOP		0x00400000	/* stop background jobs from output */
#define	FLUSHO		0x00800000	/* output being flushed (state) */
#define	NOKERNINFO	0x02000000	/* no kernel output from VSTATUS */
#define	PENDIN		0x20000000	/* XXX retype pending input (state) */
#define	NOFLSH		0x80000000	/* don't flush after interrupt */


typedef unsigned long	tcflag_t;
typedef unsigned char	cc_t;
typedef long		speed_t;

struct termios {
	tcflag_t	c_iflag;	/* input flags */
	tcflag_t	c_oflag;	/* output flags */
	tcflag_t	c_cflag;	/* control flags */
	tcflag_t	c_lflag;	/* local flags */
	cc_t		c_cc[NCCS];	/* control chars */
	long		c_ispeed;	/* input speed */
	long		c_ospeed;	/* output speed */
};

/* 
 * Commands passed to tcsetattr() for setting the termios structure.
 */
#define	TCSANOW		0		/* make change immediate */
#define	TCSADRAIN	1		/* drain output, then change */
#define	TCSAFLUSH	2		/* drain output, flush input */
#define	TCSASOFT	0x10		/* flag - don't alter h.w. state */


/*
 * Standard speeds
 */
#define	B0	0
#define	B50	50
#define	B75	75
#define	B110	110
#define	B134	134
#define	B150	150
#define	B200	200
#define	B300	300
#define	B600	600
#define	B1200	1200
#define	B1800	1800
#define	B2400	2400
#define	B4800	4800
#define	B9600	9600
#define	B19200	19200
#define	B38400	38400
#define	EXTA	19200
#define	EXTB	38400
#define	B57600	57600
#define	B115200	115200


speed_t	cfgetispeed (const struct termios *);
speed_t	cfgetospeed (const struct termios *);
int	cfsetispeed (struct termios *, speed_t);
int	cfsetospeed (struct termios *, speed_t);
int	tcdrain (int);
int	tcflow (int, int);
int	tcflush (int, int);
int	tcgetattr (int, struct termios *);
int	tcsendbreak (int, int);
int	tcsetattr (int, int, const struct termios *);

#define	TCIFLUSH	1
#define	TCOFLUSH	2
#define	TCIOFLUSH	3
#define	TCOOFF		1
#define	TCOON		2
#define	TCIOFF		3
#define	TCION		4

void	cfmakeraw (struct termios *);
void	cfsetspeed (struct termios *, speed_t);

#endif /* !_SYS_TERMIOS_H_ */


#ifndef _POSIX_SOURCE
#include <sys/ttydefaults.h>
#endif  /*_POSIX_SOURCE */

